# A simple script to scrape prices from Yahoo
# 31Jul15 Modified regex to allow for non-numeric return from Yahoo

#import urllib.parse
import re
import sys
import time
if sys.version[0:1] == '3':
	import urllib.request
else:
	import urllib

from com.sun.star.awt import Rectangle
from com.sun.star.awt import WindowDescriptor
from com.sun.star.awt.WindowClass import MODALTOP
from com.sun.star.awt.VclWindowPeerAttribute import OK, OK_CANCEL, YES_NO, YES_NO_CANCEL, RETRY_CANCEL, DEF_OK, DEF_CANCEL, DEF_RETRY, DEF_YES, DEF_NO
from com.sun.star.awt.FontWeight import BOLD, NORMAL

# Message box test for OO or LO version
# Uses either messageBoxOO4 or messageBoxLO4. Pretty much pot luck which one works, depending
# on which version of OpenOffice or LibreOffice is used
def messageBox(msgText, msgTitle):
	doc = XSCRIPTCONTEXT.getDocument()
	parentwin = doc.CurrentController.Frame.ContainerWindow

	try:
		messageBoxLO4(parentwin, msgText, msgTitle)
	except:
		messageBoxOO4(parentwin, msgText, msgTitle)
	else:
		pass

### following works with OO4 but not LO4 or OO Portable 3.2
def messageBoxOO4(ParentWin, MsgText, MsgTitle):
	#doc = XSCRIPTCONTEXT.getDocument()
	ctx = XSCRIPTCONTEXT.getComponentContext()
	MsgText = MsgText+"      OO"
	
	toolkit = ctx.getServiceManager().createInstanceWithContext("com.sun.star.awt.Toolkit", ctx)
	msgbox = toolkit.createMessageBox(ParentWin, 0, 1, MsgTitle, MsgText)
    
	msgbox.execute()
	#msgbox.dispose()

# Show a message box with the UNO based toolkit. Works with LO4 and OOP3.2
# But.... does not work with LO 4.2.3.3
def messageBoxLO4(ParentWin, MsgText, MsgTitle):
	MsgText = MsgText+"      LO"
	#describe window properties.
	aDescriptor = WindowDescriptor()
	aDescriptor.Type = 1   					# MODALTOP
	aDescriptor.WindowServiceName = "infobox"
	aDescriptor.ParentIndex = -1
	aDescriptor.Parent = ParentWin
	aDescriptor.WindowAttributes = 4194304	#MsgButton = OK
	
	tk = ParentWin.getToolkit()
	msgbox = tk.createWindow(aDescriptor)
	
	msgbox.setMessageText(MsgText)
	msgbox.setCaptionText(MsgTitle)
		
	return msgbox.execute()



#================================================================================
#================================================================================	
	
def UpdateYahooPricePy():	
	# This is the main function to start the script
	oDoc = XSCRIPTCONTEXT.getDocument()
	oSheets = oDoc.getSheets()
	mySheet = oSheets.getByName("Sheet1")
	parentwin = oDoc.CurrentController.Frame.ContainerWindow
	
	# delete existing prices and build list of epics
	urlMiddle = ''
	StockRow = 1
	while True:
		if len(mySheet.getCellByPosition(0,StockRow).String) == 0:
			break
		mySheet.getCellByPosition(1,StockRow).clearContents(5)
		urlMiddle = urlMiddle + mySheet.getCellByPosition(0,StockRow).String + '+'
		StockRow += 1
	
	urlMiddle = urlMiddle[:len(urlMiddle)-1]		# trim final +	
	
	# get prices from Yahoo
	url = 'http://quote.yahoo.com/d/quotes.csv?s=' + urlMiddle + '&f=sl1&e=.csv'
	
	if sys.version[0:1] == '3':
		response = urllib.request.urlopen(url)
	else:
		response = urllib.urlopen(url)
		
	html = response.read()
	html = html.decode('1252','ignore')

	reg = re.compile("[\n]")
	html = reg.sub('',html)        # remove tabs, LF/CR
	html = html + '"'
	
	# write prices to current_status sheet
	StockRow = 1
	while True:
		#matchObj = re.search( r'.*?(.*?),(\d*\.\d*)(.*)', html)
		#matchObj = re.search( r'.*?(.*?),(\d*\.\d*|N/A)(.*)', html)
		matchObj = re.search( r'"(.*?)",(.*?)(".*)', html)
		if not matchObj:
			break
		html = matchObj.group(3).strip()
		
		# if cannot convert price to float, then must be N/A or some such thing
		try:
			mySheet.getCellByPosition(1,StockRow).Value = float(matchObj.group(2))
		except:
			mySheet.getCellByPosition(1,StockRow).Formula = '=NA()'
		
		
		#mySheet.getCellByPosition(1,StockRow).Value = price
		StockRow += 1
	
	messageBox("Processing finished", "Status")

#=========================================================================
#Define which functions are visible in My Macros
g_exportedScripts = UpdateYahooPricePy, 